effect.plot <- function(fit,effect=effect.fit(fit,adjust=adjust,...),
   adjust=T,boxplot=10,jitter=.1,width=jitter,xlim=xlims,ylim=ylims,xaxt="i",
   xlab="Terms",ylab=ylabs,...)
{
   nterms <- length(effect)
   # eliminate degenerate effects
   for (i in 1:nterms)
      if (all(is.na(effect[[i]])))
         effect[[i]] <- NULL
   nterms <- length(effect)

   if (adjust)
      ylabs <- "MS Adjusted Effects"
   else
      ylabs <- "Effects"

   xlims <- c(1-max(jitter,width),nterms+max(jitter,width))
   ylims <- range(unlist(effect))
   plot(xlim,ylim,type="n",xaxt="n",xlab=xlab,ylab=ylab,...)
   if( xaxt != "n" )
      axis(1,1:nterms,names(effect))
   abline(h=0,lty=2)
   par(lty=1)

   for (i in 1:nterms) {
      neff <- length(effect[[i]])
      if (neff < boxplot) {
         name1 <- names(effect[[i]])
         if(is.null(name1))
            points(i+runif(neff,-jitter,jitter),effect[[i]])
         else
            text(i+runif(neff,-jitter,jitter),effect[[i]],labels=name1)
            
      }
      else
         box.fig(effect[[i]],i,width,jitter)
   }
   invisible(effect)
}


# effects contrasts

effect.fit <- function(object, ...) UseMethod("effect.fit")

effect.fit.default <- function(fit,data=eval(fit$call$data),
   factors=all.factors(fit,data),adjust=T,influence=F,
   terms.fit=as.data.frame(predict(fit,type="terms")),...)
{
   if (adjust)
      tmp <- function(x,f=NULL) {
         if (is.null(f)) {
            nx <- tapply(x,x,length)
            ux <- as.numeric(names(nx))
            ff <- NULL
         }
         else {
            nx <- tapply(x,f,length)
            ux <- tapply(x,f,mean)
            ff <- tapply(f,f,unique)
            if (is.factor(f))
               ff <- levels(f)[ff]
         }
         lx <- length(ux)
         x <- ux * sqrt(nx*lx/(lx-1))
         list(x=x,f=ff)
      }
   else
      tmp <- function(x,f=NULL) {
         if(is.null(f)) {
            x <- unique(x)
            ff <- NULL
         }
         else {
            x <- tapply(x,f,mean)
            ff <- tapply(f,f,unique)
            if (is.factor(f))
               ff <- levels(f)[ff]
         }
         list(x=x,f=ff)
      }

   se <- .01*diff(range(terms.fit))
   eff <- list()
   for (i in names(terms.fit)) {
      tmpterm <- tmp(precision(terms.fit[[i]],se),data[[i]])
      eff[[i]] <- tmpterm$x
      if (!is.na(match(i,factors)))
         names(eff[[i]]) <- as.character(tmpterm$f)
      else
         names(eff[[i]]) <- NULL
   }

   tmp <- resid(fit)
   if (influence)
      eff[["resid"]] <- tmp / sqrt(1-lm.influence(fit)$hat)
   else {
      if (adjust)
         eff[["resid"]] <- tmp * sqrt(length(tmp)/df.resid(fit))
      else
         eff[["resid"]] <- tmp
   }
   eff
}

effect.fit.listof <- function(fit,data=eval(attr(fit,"call")$data),
   factors=all.factors(fit,data),terms.fit=fit.proj,stratum,...)
{
   fit.proj <- as.data.frame(proj(fit)[[stratum]])
   fit.proj$Residuals <- NULL
   effect.fit.default(fit[[stratum]],data=data,factors=factors,
      terms.fit=terms.fit,...)
}
