circle <- function(x=0,y=0,radius=1,sx=radius,sy=radius,...)
   invisible(ellipse(x,y,sx=sx,sy=sy,...))

ellipse <- function(x=0,y=0,sx=1,sy=1,rho=0,fine=.0125,center=.1,show=T,add=T,
                        orient = 0, xo = 0, yo = 0,
                        xlab="",ylab="",bty="n",xaxt="n",yaxt="n",type="n",lty=1,pty="s",err=-1,
                        ...)
{
   if(min(sx) < 0 | min(sy) < 0 | max(abs(rho)) > 1)
      stop("sx,sy or rho out of bounds")
   if(fine <= 0 | fine > 1)
      stop("fine out of bounds")
   n <- max(length(x),length(y),length(sx),length(sy),length(rho),length(orient),length(xo),length(yo))
   x <- array(x,n)
   y <- array(y,n)
   lty <- array(lty,n)
   sx <- array(sx,n)
   sy <- array(sy,n)
   rho <- array(rho,n)
   center <- array(center,n)
   prin1 <- sqrt((1+rho)/2)
   prin2 <- sqrt((1-rho)/2)
   xarc <- sin(pi * seq(0,.5,by=4*fine))
   yarc <- c(rev(xarc),-xarc)
   xarc <- c(xarc,yarc,-rev(xarc))
   yarc <- c(yarc,rev(yarc))
   orient <- array(orient,n)
   xo <- array(xo,n)
   yo <- array(yo,n)

   costheta <- cos( orient )
   sintheta <- sin( orient )

   arc <- list()
   for (i in 1:n)
   {
      tmpx <- x[i] + sx[i]*(prin1[i]*xarc+prin2[i]*yarc)
      tmpy <- y[i] + sy[i]*(prin1[i]*xarc-prin2[i]*yarc)
      arc[[i]] <- data.frame(
         x = xo[i] + costheta[i] * tmpx + sintheta[i] * tmpy,
         y = yo[i] + costheta[i] * tmpy - sintheta[i] * tmpx )
   }
   if(show) {
      if(!add)
      {
         tmp <- as.matrix( as.data.frame( lapply( arc,
            function( a ) c( range( a$x ), range( a$y )))))
         plot( c(tmp[1:2,]), c(tmp[3:4,]),
            bty=bty,xaxt=xaxt,yaxt=yaxt,type="n",xlab=xlab,ylab=ylab,pty=pty,
            ...)
      }
      for (i in 1:n) {
         lines(arc[[i]]$x,arc[[i]]$y,lty=lty[i],err=err,...)
         if(center[i] > 0) {
            lines(x[i]+center[i]*c(-1,1)*sx[i]*prin1[i],
               y[i]+center[i]*c(-1,1)*sy[i]*prin1[i],err=err)
            lines(x[i]+center[i]*c(-1,1)*sx[i]*prin2[i],
               y[i]+center[i]*c(1,-1)*sy[i]*prin2[i],err=err)
         }
      }
   }
   invisible(arc)
}
norm.ellipse <- function(x, y, group, homosked=T, level=.05,
   radius=sqrt(qchisq(1-level,2)),...) {
   mx <- mean(x)
   my <- mean(y)
   sx <- std.dev(x)
   sy <- std.dev(y)
   rho <- if(length(x) > 1) cor(x,y,trim=.00001) else 1
   ellipse(mx,my,sx*radius,sy*radius,rho,...)
   invisible(list(x=c(mx,sx),y=c(my,sy),rho=rho))
}
