df.resid <- function(object, ...) UseMethod("df.resid")

df.resid.default <- function(...)
{
   x <- list(...)
   if(length(x) == 1 && is.list(x[[1]]))
      x <- x[[1]]
   x <- tapply(x[[1]],x,length) - 1
   x[is.na(x)] <- 0
   sum(x)
}

df.resid.lm <- function(object,resid=residuals(object))
{
   rdf <- object$df.resid
   if(is.null(rdf)) {
      p <- object$rank
      if(is.null(p))
         p <- sum(!is.na(coefficients(object)))
      rdf <- length(resid) - p
   }
   rdf
}
se.coef <- function(object,summary=summary.lm(object))
{
   summary$coefficients[,2]
}

cor.coef <- function(object,summary=summary.lm(object))
{
   summary$correlation
}

cov.coef <- function(object,summary=summary.lm(object))
{
   summary$cov.unscaled
}
std.dev <- function(object)
{
   # adapted from summary.lm
   sd <- summary(object)$sigma
   if(is.null(sd))
   {
      resid <- residuals(object)
      if(is.null(resid))
         sd <- sqrt(var(object))
      else
      {
         rdf <- df.resid(object)
         if(rdf > 0)
            sd <- sqrt(sum(resid^2)/rdf)
         else
            sd <- NA
      }
   }
   if(!is.na(sd)) sd
      else 0
}
#std.err <- function( fit, data = eval( fit$call$data ),
   #factors = all.factors( fit, data ))
#{
   #std.dev( fit ) / sqrt( harmonic.mean( sample.size(
   #fit, data, factors )))
#}

sample.size <- function(object, ...) UseMethod("sample.size")

sample.size.default <- function(...)
{
   x <- list(...)
   if(length(x) == 1 && is.list(x[[1]]))
      x <- x[[1]]
   x <- tapply(x[[1]],x,length)
   x[is.na(x)] <- 0
   x
}

sample.size.lm <- function(fit,data=eval(fit$call$data),
   factors=all.factors(fit,data),na.action=na.omit)
{
   n <- length(factors)
   if (n < 1)
      length(data[[model.response(fit)]])
   else {
      replications(formula(paste("~",paste(factors,collapse=":"),sep="")),
         data, na.action = na.action)[[1]]
   }
}
cor.coef <- function(object)
{
   summary.lm(object)$correlation
}
harmonic.mean <- function(x)
{
	length(x)/sum(1/x)
}
