.BG
.FN lsd.plot
.FN ci.plot
.FN int.plot
.TL
Interaction Plots of Response with Interval Bars
.CS
lsd.plot(x.factor, response, group, ...)
ci.plot(x.factor, response, group, ...)
int.plot(x.factor, response, group, ..., bar.plot="lsd")

lsd.plot(fit, data, factors, ...)
ci.plot(fit, data, factors, ...)
int.plot(fit, data, factors, lsm=lsmean(fit,data,factors),
   sort.mean=F, ..., bar.plot=c("lsd","ci","test","none"))
.AG x.factor
factor to be plotted on the \f2x\f1-axis.
.AG response
numeric vector.
.AG sort.mean
should x factor levels be sorted by mean value (default=F).
.AG group
factor whose levels will be separate traces.
.AG fit
fitted model object (optional unless response is omitted).
.AG data
data frame in which to interpret the variables named in
the fit object (taken from fit if omitted).
.AG factors
Character string of length 2 with names of x.factor and trace.factor
as found in fit and data (default is first 2 factors in fit).
.AG lsm
Least squares means for plotting (default is fit from model).
.AG ...
optional specification of other parameters to be passed to plotting routines
and/or to the bar-drawing routines `lsd.bar' and `se.bar'.
There are some parameters, such as `width' and `edge' which can change to
plotting area.
.AG bar.plot
"lsd" (=default), "ci", "test" or "none".
The "test" is a sqrt(2) contraction of confidence interval for approximate
graphical comparisons of means.
.PP
NOTE:
The arguments for the default calls agree in order
with `mplot' but not `interaction.plot'.
The routines `lsd.plot' and `ci.plot' simply call `int.plot'.
.PP
A plot will be created showing the requested function of responses 
for each level of the `x.factor' at each level of the `group'.
By default, lines for each value of the `group' are drawn in different
line styles ('lsd.plot' and 'int.plot') and with plot symbols as the first
character of `group' so
that they may be more easily distinguished.  This corresponds to plot option
type="b", which can be overridden.
A Fisher's LSD bar (lsd.plot) or confidence intervals (ci.plot) are added.
.RT
list containing sd, rdf, level and sample size, and either lsd (lsd.plot)
or width (ci.plot).
.Xp D2way
.SA
`interaction.plot', `margin.plot.lm', `lsd.bar', `ci.width', `mplot'.
.EX
> # the first use mimics interaction.plot
> attach(catalyst)
> lsd.plot(Conc, Yield, Cat)
> detach()
> # or you could get more fancy by using a model fit
> # here is an interaction plot averaged over Conc levels
> cat.fit <- aov(Yield~Conc+Cat*Temp,catalyst)
> lsd.plot(cat.fit,catalyst,factors=c("Temp","Cat"))
> # the following gives a plot only for one level of Conc
> cat.lsm <- lsmean(cat.fit)
> lsd.plot(cat.fit,catalyst,factors=c("Temp","Cat"),
     lsm=cat.lsm[cat.lsm$Conc==20,])
> # try these using ci.plot instead of lsd.plot
.KW design
.WR
