lsd.plot <- function(object, ...) UseMethod("int.plot")
int.plot <- function(object, ...) UseMethod("int.plot")

int.plot.default <- function(x.factor,response,group,
   xlab=xlabs,ylab=ylabs,...)
{
   xlabs <- deparse(substitute(x.factor))
   ylabs <- deparse(substitute(response))

   if (!is.factor(x.factor))
      x.factor <- factor(x.factor)
   if( missing(group) ) {
      data <- data.frame(response,x.factor)
      names(data) <- c(ylabs,xlabs)
      fit <- lm(paste(ylabs,xlabs,sep="~"),data)
   }
   else {
      glabs <- deparse(substitute(group))
      if (!is.factor(group))
         group <- factor(group)
      data <- data.frame(response,x.factor,group)
      names(data) <- c(ylabs,xlabs,glabs)
      fit <- lm(paste(ylabs,".^2",sep="~"),data)
   }
   invisible(int.plot.lm(fit,data,xlab=xlab,ylab=ylab,...))
}

int.plot.lm <- function(fit,data=eval(fit$call$data),
   factors=all.factors(fit,data),lsm=lsmean(fit,data,factors),
   ci=ci.width(fit,data,factors,lsm),offset=0.01,bar.plot="lsd",
   xlim=xlims,ylim=ylims,type="b",width=.02,sort.mean=F,edge=wi,
   xlab=xlabs,ylab=model.response(fit),xaxt="s",...)
{
   # what type of bar plot?
   bar.plot <- pmatch(bar.plot,c("lsd","ci","test"),nomatch=0)

   # make room for confidence intervals or test intervals
   if (bar.plot > 1) {
      wi <- .02
      if (bar.plot == 3)
         ci <- ci / sqrt(2)
      ylims <- c(min(lsm$pred-ci),max(lsm$pred+ci))
   }
   else {
      wi <- 0
      ylims <- range(lsm$pred)
   }

   # set up plotting groups if any
   xlabs <- factors[1]
   lf <- length(factors)
   if (lf == 1) { # only 1 factor
      group <- NULL
      x.factor <- lsm[[factors]]
   }
   else {
      if (lf > 2) { # x is 1st factor, all others have traces
         group <- interaction(get.list(lsm,factors[-1]))
         x.factor <- lsm[[factors[1]]]
      }
      else { # list of two vectors of factor names
         group <- interaction(get.list(lsm,factors[[2]]))
         x.factor <- interaction(get.list(lsm,factors[[1]]))
         xlabs <- paste(factors[[1]],collapse=".")
   }
}
   # set up horizontal axis -- factor levels or sorted by mean
   if (sort.mean) {
      if (is.null(group))
         x <- order(order(-lsm$pred))
      else {
         x <- order(order(-tapply(lsm$pred,x.factor,mean)))
         names(x) <- levels(x.factor)
         x <- x[x.factor]
      }
   }
   else
      x <- unfactor(x.factor)
   # horizontal limits
   xlims <- range(x)
   if (width+edge > 0)
      xlims <- xlims * (1+width+edge) - rev(xlims) * (width+edge)

   # multiple group plot
   mplot(x,lsm$pred,group,xlim=xlim,ylim=ylim,xaxt="n",type=type,
      xlab=xlab,ylab=ylab,...)
   # add factor levels on horizontal axis
   if (xaxt != "n")
      mtext(as.character(x.factor),1,at=x)

   # intervals at each point or one LSD bar or none?
   if (bar.plot) {
      if (bar.plot > 1) {
         x <- unfactor(x,"ordered")
         group <- if (length(factors) == 2) lsm[[factors[2]]] else NULL
         if (!is.null(group) & offset > 0)
            x <- x + offset * diff(par("usr"))[1] *
               (unclass(group) - (1+length(levels(group)))/2)

         invisible(se.bar(x,lsm$pred,2*ci,width=width,cap="",...))
      }
      else
         invisible(lsd.bar.lm(fit,data,factors,lsm=lsm,width=width,...))
   }
}
