lsmean <- function(object, ...) UseMethod("lsmean")

lsmean.default <- function(y,...,factors=all.factors(fit,data)[1:2],
   effects=F,se.fit=T,adjust.covar=T)
{
   data <- eval(substitute(data.frame(y,...)))
   y <- deparse(substitute(y))
   fit <- lm(paste(y,".",sep="~"),data,singular.ok=T)
   lsmean.lm(fit,data,factors,effects=effects,se.fit=se.fit,adjust.covar=adjust.covar)
}

lsmean.lm <- function(fit,data=eval(fit$call$data),
   factors=names(predictors[predictors]),expr=formula(fit),
   contrast=fit$contrasts,effects=F,se.fit=T,adjust.covar=T)
{
   expr.labels <- attr(terms(expr),"term.labels")
   # get predictors from right side of equation
   predictors <- unlist(lapply(data, is.factor))[
      all.names(expr, unique = T, functions = F)[-1]]
   predictors <- predictors[!is.na(charmatch(names(predictors),expr.labels))]

   # set up factors of interest, auxiliary factors and covariates
   covars <- names(predictors[!predictors])
   aux.factors <- predictors[predictors]
   if (length(aux.factors)) {
      pm <- pmatch(unlist(factors),names(aux.factors))
      pm <- pm[!is.na(pm)]
   }
   else
      pm <- numeric(0)
   if(length(pm)==0) {
      factors <- NULL
      aux.factors <- names(aux.factors)
   }
   else {
      factors <- names(aux.factors[pm])
      aux.factors <- names(aux.factors[-pm])

      # check contrasts if using effects option
      if (effects) {
         for (i in factors) if (any(abs(apply(contrast[[i]],2,sum)>1e-12)))
            stop(paste("effects=T not implemented for factors",
               "without sum-to-zero contrasts (",i,")"))
      }
   }

   # separate auxiliary factors with respect to sum-to-zero contrasts
   cont.factors <- NULL
   for (i in aux.factors) if (any(apply(contrasts(data[[i]]),2,sum)))
      cont.factors[i] <- i
   if(length(cont.factors)) if(!is.null(aux.factors))
      aux.factors <- aux.factors[-pmatch(aux.factors,cont.factors,nomatch=0)]

   # ignore aux factors whose contrasts sum to zero
   fac <- attr(terms(expr),"factors")
   tmp <- c(factors,covars,cont.factors)
   term.factors <- fac[tmp,]
   if (length(tmp) > 1)
      term.factors <- apply(term.factors,2,any)
   not.ignore <- 1 - fac[aux.factors,]
   if (length(aux.factors) > 1)
      not.ignore <- apply(not.ignore,2,all)
   if (length(aux.factors) & length(not.ignore))
      term.factors <- term.factors & not.ignore
   term.factors <- expr.labels[term.factors]

   # get indexes for terms used in LS mean fit
   coef <- coefficients(fit)
   pos <- attr(coef,"assign")
   index <- if (!effects) pos[["(Intercept)"]]
   for (i in term.factors)
      index <- c(index,pos[[i]])

   # simplify design: unique factor combinations, covariates at mean level

   # unique factor combinations
   if(is.null(factors)) {
      newdata <- data[1,]
      int.factors <- "mean"
      row.names(newdata) <- int.factors
   }
   else {
      int.factors <- interaction(data[,factors],drop=T)
      design <- !duplicated(int.factors)
      newdata <- data[design,]
   }

   # average covariates to mean value
   if (adjust.covar | is.null(factors))
      for (i in covars)
         newdata[,i] <- mean(data[,i])
   else
      for (i in covars)
         newdata[,i] <- tapply(data[,i],int.factors,mean)

   if(!is.null(factors)) {
      for (i in rev(factors))
         newdata <- newdata[order(newdata[[i]]),]
      row.names(newdata) <- int.factors <- interaction(newdata[,factors],drop=T)
   }

   # expand matrix for auxiliary factors without sum-to-zero contrasts
   if (!is.null(cont.factors)) {
      cont.levels <- expand.grid(get.list(factor.names(data),cont.factors))
      cont.n <- nrow(cont.levels)
      newdata <- newdata[rep(int.factors,cont.n),]
      tmp <- rep(seq(len=cont.n),rep(length(int.factors),cont.n))
      newdata[,cont.factors] <- cont.levels[tmp,]
   }

   # THIS BREAKS FOR NESTED MODELS (in model.frame)
   # SEE lsmean.listof() below for a kludge fix
   mat <- t(as.matrix(model.matrix.default(expr,newdata,contrast)[,index]))
   # average over levels for auxiliary factors without sum-to-zero contrasts
   if (!is.null(cont.factors)) {
      tmp <- array(mat,c(nrow(mat),ncol(mat)/cont.n,cont.n))
      dimnames(tmp) <- list(dimnames(mat)[[1]],NULL,NULL)
      mat <- apply(tmp,c(1,2),mean)
   }
   mat <- as.matrix(mat)
   # predicted values (THIS IS BROKEN IF THERE ARE EMPTY CELLS!)
   pred <- coef[index] * mat
   if (length(index) == 1)
      pred <- unlist(pred)
   else
      pred <- apply(pred,2,sum)
   # standard error
   if(se.fit) {
      cov <- cov.coef(fit)[index,index]
      if(length(index) == 1) 
         se <- sqrt(cov) * abs(mat)
      else
         se <- sqrt(apply(mat*(cov%*%mat),2,sum))
      se <- std.dev(fit) * se
   }
   if(is.null(factors) & is.null(covars))
      newdata <- data.frame(pred=matrix(pred,1,1),row.names=int.factors)
   else {
      newdata <- as.data.frame(newdata[int.factors,c(factors,covars)])
      dimnames(newdata) <- list(int.factors,c(factors,covars))
      names(pred) <- as.character(int.factors)
      newdata$pred <- pred
   }
   if(se.fit) {
      if(is.null(factors) & is.null(covars))
         se <- matrix(se,1,1)
      newdata$se <- se
      names(newdata$se) <- row.names(newdata)
   }
   newdata
}

lsmean.listof <- function(fit,data=eval(attr(fit,"call")$data),
   factors=names(fit[[stratum]]$contrasts),stratum,expr=formula(fit),
   contrast=fit.cont,...)
{
   if(missing(stratum))
	stop("must specify stratum for fit of nested model")
   fit.cont <- NULL
   for (i in names(fit))
      if (i != "(Intercept)")
      fit.cont <- c(fit.cont,fit[[i]]$contrasts)
   lsmean.lm(fit[[stratum]],data=data,factors=factors,expr=expr,
      contrast=contrast,...)
}
