margin.plot <- function(object, ...) UseMethod("margin.plot")

margin.plot.default <- function(x.factor,response,group,
   orient = "default", average = "full",
   xlab=xlabs,ylab=ylabs,...)
{
   xlabs <- deparse(substitute(x.factor))
   xlabs[2] <- deparse(substitute(group))
   ylabs <- deparse(substitute(response))

   data <- data.frame(response,x.factor,group)
   names(data) <- c(ylabs,xlabs)

   reduced <- lm(paste(ylabs,".",sep="~"),data)
   full <- update(reduced,~.^2,singular.ok=T)
 
  if(is.na(pmatch(average,"full")))
      lsm <- lsmean(reduced,data)
   else
      lsm <- lsmean(full,data)

   orient <- pmatch(orient,c("switch","both"),nomatch=0)
   if (orient == 1)
      ylabs <- c("",ylabs)
   if(orient != 1)
      ret <- margin.plot.lm(full,reduced,data,factors=xlabs,lsm=lsm,
	 xlab=xlab[1],ylab=ylab[1],...)
   if(orient > 0)
      ret <- margin.plot.lm(full,reduced,data,factors=rev(xlabs),lsm=lsm,
         xlab=xlab[2],ylab=ylab[2],...)
   invisible(list(ret=ret,reduced=reduced,full=full))
}

margin.plot.lm <- function(full,reduced=full,
   data=eval(full$call$data),factors=all.factors(full,data),
   lsm=lsmean(full,data,factors),margin=margin.labels,error="full",
   bar.plot="lsd",xlab=xlabs,ylab="",xlim=xlims,ylim=ylims,
   pty="s",xaxt="s",...)
{
   # allow for multi-factor horizontal axis
   x.factor <- factors[[1]]
   xlabs <- paste(x.factor,collapse=".")
   factors[[1]] <- xlabs
   x <- interaction(get.list(lsm,xlabs))
   ylims <- range(lsm$pred)
   xlims <- range(unfactor(lsm[[x.factor[1]]]))

   # use margin values if they correspond to levels of first factor
   # lazy evaluation: margin=margin.labels
   margin.labels <- precision(lsmean(reduced,data,x.factor))
   if (margin[1]) {
      names(margin) <- names(margin.labels)
      lsm[[xlabs]] <- ordered(margin[as.character(x)])
      tmpar <- par(pty=pty)
      xlims <- ylim
   }
   if(is.na(pmatch(error,"full"))) {
      error <- reduced
      lsm$se <- lsmean(reduced,data,factors)$se
   }
   else
      error <- full
   ret <- int.plot(error,data,factors,lsm=lsm,xlab=xlab,ylab=ylab,
      xlim=xlim,ylim=ylim,bar.plot=bar.plot,xaxt=xaxt,...)

   # add actual factor 1 levels if using marginal mean values 
   if (margin[1]) {
      if (xaxt != "n") {
         mtext(names(margin),1,1,at=margin)
         right <- par("usr")[1]
         mtext(xlabs,1,1,at=right)
         mtext("margin",1,0,at=right)
      }
      par(tmpar)
   }
   ret$par <- tmpar
   invisible(ret)
}
