multi.table <- function( pred, se, rdf, method = "DUN", alpha = .05,
                        harmonic = T, qvalue = qstud( npred, rdf, alpha, method ))
  {
    method <- multi.comp( method )

    o <- order( -pred )
    se2 <- se[o] ^ 2
    if( harmonic )
      se2 <- rep( mean( se2 ), length( se2 ))
    pred <- pred[o]
    npred <- length( pred )
    dif <- outer( pred, pred, "-" )
    se <- sqrt( outer( se2, se2, "+" ))
    nom <- names( pred )
    switch( method,
           "LSD" = { z <- 2 * pt( -abs( dif / se ), rdf )
                   },
           "BON" = { z <- pt( -abs( dif / se ), rdf ) *
                       npred * ( npred - 1 )
                   },
           "SNK" = { k <- 1 + abs( outer( seq( npred ), seq( npred ), "-" ))
                     k[ k == 1 ] <- 2
                     z <- as.numeric( sqrt(2) * abs( dif / se ) < qvalue[k-1] )
                   },
           "DUN" = {k <- 1 + abs( outer( seq( npred ), seq( npred ), "-" ))
                    k[ k == 1 ] <- 2
                    z <- as.numeric( sqrt(2) * abs( dif / se ) < qvalue[k-1] )
                  })
    array( z, dim( dif ), dimnames = list( nom, nom ))
  }
qstud <- function( n, rdf, alpha = 0.05, method = "DUN" )
  {
    q <- numeric(0)
    switch( method,
           "DUN" = {
             aa <- 1
             a <- 1 - alpha
             for( i in seq( n - 1 ))
               {
                 aa <- aa * a
                 q[i] <- qtukey( aa, i + 1, rdf )
               }
           },
           "SNK" = {
             aa <- 1 - alpha
             for( i in seq( n - 1 ))
               q[i] <- qtukey( aa, i + 1, rdf )
           })
    q
  }
multi.comp <- function( method = "DUN" )
  {
    # force LSD for SPlus Version 3
    if( version$major < 4 )
      method <- "LSD"
    else
      {
        multcomps <- c("LSD","DUN","BON","SNK")
        method <- multcomps[ pmatch( method, multcomps, nomatch = 1 ) ]
      }
    method
  }
