pletter <- function( ptable, alpha = .05, let = letters )
  {
    n <- nrow( ptable )
    ptable[ seq( from = 1, to = n*n, by = n+1 )] <- 1
    for( i in seq( n - 1 ))
      ptable[ i, seq( i + 1, n ) ] <- 0

    ptable <- ptable > alpha
    ptable[ is.na( ptable ) ] <- T
    prev <- ptable[,1]

    code <- character( n )
    code[ prev ] <- let[1]
    j <- 1
    for( i in seq( 2, n )) if( any( (!prev) & ptable[,i] ))
      {
        j <- j + 1
        prev <- ptable[,i]
        code[prev] <- paste( code[prev], let[j], sep = "" )
      }
    names(code) <- dimnames( ptable )[[1]]
    code
  }
pletter.lohi <- function( plet )
  {
    x <- as.data.frame( lapply( pletter.list( plet ), range ))
    sprod <- seq( plet )
    lohi <- data.frame( lo = sprod, hi = sprod, row.names = names( plet ))
    sx <- seq( ncol( x ))
    for( i in sprod )
      {
        tmp <- sx[ i >= x[1,] & i <= x[2,] ]
        lohi[i,] <- c( x[1,min(tmp)], x[2,max(tmp)] )
      }
    lohi
  }
pletter.list <- function( plet )
  {
    plet <- as.character( plet )
    nprod <- length( plet )
    x <- list( )
    i <- 1
    li <- letters[i]
    sprod <- seq( nprod )
    pplet <- paste( plet, collapse = "" )

    while( regexpr( paste( "[", li, "-z]", sep = "" ), pplet ) > 0 )
      {
        tmp <- sprod[ regexpr( li, plet ) > 0 ]
        if( length( tmp ))
          x[[li]] <- tmp
        i <- i + 1
        li <- letters[i]
      }
    if( length( x ) == 0 )
      x <- list( sprod )
    x
  }

pvalue.ellipse <- function( plet, lsmeans, se, ddf,
                     pset = pletter.list( plet ),
                     alpha = .975, scale = 0.3, full.width = T )
  {
    ni <- length( plet )
    ns <- length( pset )
    elstuff <- matrix( NA, 2, ns, dimnames = list( c("x","sx"), 1:ns ))
    if( length( ddf ) == 1 )
      ddf <- array( ddf, ni )

    pnames <- names( lsmeans )
    if( is.null( pnames ))
      pnames <- as.character( seq( lsmeans ))

    tmp <- function( r, se, ddf, alpha, scale, full.width,
                    tmpfn = function( x, y ) sqrt( x^2 + y^2 ))
      {
        if( length( se ) > 1 )
          {
            se <- outer( se, se, tmpfn )
            se <- max( se[ row( se ) > col( se ) ] )
            radius <- scale * se + diff( r ) / 2
          }
        else
          {
            radius <- se <- 0
          }
        ddf <- min( ddf )

        if( full.width )
          c( mean( r ), max( radius, se * qt( alpha, ddf ) / 2 ))
        else
          c( mean( r ), max( radius ))
      }
    for( i in seq( ns ))
      {
        same <- pset[[i]]
        elstuff[,i] <- tmp( range( lsmeans[same] ), se[same], ddf[same],
                           alpha, scale, full.width )
      }
    as.matrix( elstuff )
  }
pvalue.oellipse <- function( lsmeans, diffs, item = "PRODUCT",
                           alpha = .05, scale = 0.2, lsd = F )
  {
    lsmeans <- as.data.frame( lsmeans )
    diffs <- as.data.frame( diffs )
    o <- order( -lsmeans$LSMEAN )
    lsmeans <- lsmeans[o,]
    inames <- as.character( lsmeans[[item]] )
    ni <- length( inames )
    items <- paste( item, 1:2, sep = "" )
    for( i in items )
      diffs[[i]] <- as.numeric( ordered( diffs[[i]], inames ))
    diffs[,items] <- t( apply( diffs[,items], 1, sort ))

    ptable <- matrix( 0, ni, ni, dimnames = list( inames, inames ))
    ptable[ seq( from = 1, to = ni*ni, by = ni+1 )] <- 1
    ptable[ diffs[[ items[2] ]] + ni * ( diffs[[ items[1] ]] - 1 ) ] <- diffs$P.T
    # now set up as logical table
    ptable <- ptable > alpha
    alpha <- 1 - alpha / 2
    ptable[ is.na( ptable ) ] <- T
    prev <- ptable[,1]
    elstuff <- matrix( NA, 2, ni, dimnames = list( c("x","sx"), 1:ni ))

    tmp <- function( prev, lsmeans, diffs, scale, items, alpha, lsd )
      {
        r <- range( lsmeans$LSMEAN[prev] )
        prev <- seq( prev )[prev]
        prev <- !( is.na( match( diffs[[ items[1] ]], prev )) |
                  is.na( match( diffs[[ items[2] ]], prev )))
        tmp <- diff( r ) / 2
        se <- diffs$SE[prev]
        if( lsd )
          c( mean( r ), max( scale * se + tmp,
                            se * qt( alpha, diffs$DDF[prev] ) / 2 ))
        else
          c( mean( r ), max( scale * se + tmp ))
      }
    elstuff[,1] <- tmp( prev, lsmeans, diffs, scale, items, alpha, lsd )
    j <- 1
    for( i in seq( 2, ni )) if( any( (!prev) & ptable[,i] ))
      {
        j <- j + 1
        prev <- ptable[,i]
        elstuff[,j] <- tmp( prev, lsmeans, diffs, scale, items, alpha, lsd )
      }
    as.matrix( elstuff[,1:j] )
  }
pvalue.ellipses <- function( lsmeans, diffs, item = "PRODUCT", time = "TIME", ... )
  {
    lsmeans <- as.data.frame( lsmeans )
    diffs <- as.data.frame( diffs )

    interaction.plot( lsmeans[[time]], lsmeans[[item]], lsmeans$LSMEAN,
                     xlab = "Time", ylab = "Mean by Product", trace.label = "Product" )

    lt <- levels( lsmeans[[time]] )
    tmp <- function( y, i )if( !is.na( y[2] ))
      ellipse( i, y[1], .2, y[2], center = 0 )
    for( i in seq( lt ))
      {
        elstuff <- pvalue.oellipse( lsmeans[ lsmeans[[time]] == lt[i], ],
                                  diffs[ diffs[[time]] == lt[i], ], item, ... )
        apply( elstuff, 2, tmp, i )
      }
  }
interaction.ellipse <- function( lsmeans, diffs, attrib = "ATTRIB", ... )
  {
    graphsheet( orientation = "portrait" )
    par( mfrow = c(3,2), mar = c(5.1,5.1,5.1,1.1) )
    for( i in levels( lsmeans$ATTRIB ))
      {
        pvalue.ellipses( lsmeans[ lsmeans$ATTRIB == i, ],
                        diffs[ diffs$ATTRIB == i, ], ... )
        title( i )
      }
  }
