.BG
.FN se.bar
.FN lsd.bar
.TL
Standard Error and Fisher's LSD Bars for Interaction Plots
.CS
se.bar(xpos, ypos, bar, mod = "", cap = "SE", adj = 0, width = .01)
lsd.bar(x.factor, response, group, ...)
lsd.bar(fit, data, factors, xpos, ypos, ...,
	size = sample.size(fit,data,factors), rdf = df.resid(fit),
	sd = std.dev(fit), level = .05,
	critical.value = sqrt(2) * qt(1-level/2,rdf),
	cap = paste(100*level,"%\nLSD",sep=""))
.AG xpos
x position(s) of bar(s).
.AG ypos
y position(s) of bar(s).
.AG bar
height(s) of bar(s).
.AG mod
modifier to bar caption ("*" used in lsd.bar for unequal sample sizes)
.AG cap
caption for each bar (omitted if "")
.AG adj
0 = left adjust caption to right of bar,
1 = right adjust caption to left of bar.
.AG width
relative width of bar ends (can be 0).
.AG x.factor
factor to be plotted on the \f2x\f1-axis.
.AG group
factor whose levels will be separate traces.
.AG response
numeric vector.
.AG fit
fitted model object (optional unless response is omitted).
.AG data
data.frame in which to interpret the variables named in
the fit object (taken from fit if omitted).
.AG factors
Character string of length 2 with names of x.factor and trace.factor
as found in fit and data (default is first 2 factors in fit).
.AG size
sample size(s) at each bar position.
.AG rdf
residual degrees of freedom.
.AG sd
standard deviation.
.AG level
significance level.
.AG critical.value
2-sided tail value of t-distribution.
.AG ...
optional specification of other parameters to be passed to se.bar.
.PP
Standard error bars (se.bar) or an LSD bar (lsd.bar) are plotted at
position(s) cbind(xpos,ypos).  Lsd.bar uses the S method, with the
lsd.bar.default calling lsd.bar.lm after setting up the fit, data and
factor.  Lsd.bar.lm in turn calls se.bar after determining the bar
height using size, rdf, sd and level.  Confidence intervals or other
multiple bars can be constructed ci.bar (see ci.plot).
.RT
a list containg xpos, ypos, bar, mod, cap, adj, width.
.SA
`lsd.plot', `ci.plot'.
.EX
> attach(catalyst)
> lsd.bar(Conc, Cat, Yield)
> detach()
.KW design
.WR
